'use strict';

const InvalidVersionError = require('restify-errors').InvalidVersionError;
const semver = require('semver');

const SEMVER_REGEX = /v((\d+.\d+.\d+)(-[\dA-Za-z\-]+((\.[\dA-Za-z\-]+)*))?(\+([\dA-Za-z\-]+(\.[\dA-Za-z\-]+)*))?)/;

function urlSemverParser(req, res, next) {
  let apiVersion = '';
  const urlVersion = req.url.match(SEMVER_REGEX);

  req.originalUrl = req.url;

  if (urlVersion) {
    apiVersion = urlVersion[1];
    req.url = req.url.replace(SEMVER_REGEX, '').replace(/(\/)\/+/g, '$1');
  }

  apiVersion = semver.valid(apiVersion);

  if (!apiVersion) {
    return next(new InvalidVersionError('API version not supported'));
  }

  req.headers = req.headers || [];
  req.headers['accept-version'] = apiVersion;

  next();
}

module.exports = urlSemverParser;
